]> git.lyx.org Git - lyx.git/blobdiff - src/Row.h
Update Win installer for new dictionary links. Untested.
[lyx.git] / src / Row.h
index 72d7a860b73b6f890ca0c14e30b441b0c4591e31..babe11510fd1d0edb625c17476785dc230f9c054 100644 (file)
--- a/src/Row.h
+++ b/src/Row.h
@@ -51,8 +51,17 @@ public:
                INSET,
                // Some spacing described by its width, not a string
                SPACE,
-               // Something that should not happen (for error handling)
-               INVALID
+               // Spacing until the left margin, with a minimal value given
+               // by the initial width
+               MARGINSPACE
+       };
+       enum SplitType {
+               // split string to fit requested width, fail if string remains too long
+               FIT,
+               // if the requested width is too small, accept the first possible break
+               BEST_EFFORT,
+               // cut string at any place, even for languages that wrap at word delimiters
+               FORCE
        };
 
 /**
@@ -60,16 +69,11 @@ public:
  * by other methods that need to parse the Row contents.
  */
        struct Element {
-               //
-               Element() = default;
                //
                Element(Type const t, pos_type p, Font const & f, Change const & ch)
                        : type(t), pos(p), endpos(p + 1), font(f), change(ch) {}
 
 
-               // Return the number of separator in the element (only STRING type)
-               int countSeparators() const;
-
                // Return total width of element, including separator overhead
                // FIXME: Cache this value or the number of expanders?
                double full_width() const { return dim.wid + extra * countExpanders(); }
@@ -93,27 +97,24 @@ public:
                */
                pos_type x2pos(int &x) const;
                /** Break the element in two if possible, so that its width is less
-                * than \param w.
-                * \return an element containing the remainder of the text, or
-                *   an invalid element if nothing happened.
-                * \param w: the desired maximum width
-                * \param force: if true, the string is cut at any place, otherwise it
-                *   respects the row breaking rules of characters.
-                */
-               Element splitAt(int w, bool force);
-               /** Break the element if possible, so that its width is less
-                * than \param w. Returns true on success. When \param force
-                * is true, the string is cut at any place, otherwise it
-                * respects the row breaking rules of characters.
+                * than the required values.
+                * \return true if something has been done ; false if this is
+                * not needed or not possible.
+                * \param width: maximum width of the row.
+                * \param next_width: available width on next rows.
+                * \param split_type: indicate how the string should be split.
+                * \param tail: a vector of elements where the remainder of
+                *   the text will be appended (empty if nothing happened).
                 */
-               bool breakAt(int w, bool force);
+               // FIXME: ideally last parameter should be Elements&, but it is not possible.
+               bool splitAt(int width, int next_width, SplitType split_type, std::vector<Element> & tail);
+               // remove trailing spaces (useful for end of row)
+               void rtrim();
 
                //
                bool isRTL() const { return font.isVisibleRightToLeft(); }
                // This is true for virtual elements.
                bool isVirtual() const { return type == VIRTUAL; }
-               // Invalid element, for error handling
-               bool isValid() const { return type !=INVALID; }
 
                // Returns the position on left side of the element.
                pos_type left_pos() const { return isRTL() ? endpos : pos; };
@@ -121,14 +122,16 @@ public:
                pos_type right_pos() const { return isRTL() ? pos : endpos; };
 
                // The kind of row element
-               Type type = INVALID;
+               Type type;
                // position of the element in the paragraph
-               pos_type pos = 0;
+               pos_type pos;
                // first position after the element in the paragraph
-               pos_type endpos = 0;
-               // The dimension of the chunk (does not contains the
+               pos_type endpos;
+               // The dimension of the chunk (does not contain the
                // separator correction)
                Dimension dim;
+               // The width of the element without trailing spaces
+               int nspc_wid = 0;
 
                // Non-zero only if element is an inset
                Inset const * inset = nullptr;
@@ -144,7 +147,7 @@ public:
                Change change;
                // is it possible to add contents to this element?
                bool final = false;
-               // properties with respect to row breaking (made of RowFlag enums)
+               // properties with respect to row breaking (made of RowFlag enumerators)
                int row_flags = Inline;
 
                friend std::ostream & operator<<(std::ostream & os, Element const & row);
@@ -209,9 +212,9 @@ public:
        ///
        pos_type endpos() const { return end_; }
        ///
-       void right_boundary(bool b) { right_boundary_ = b; }
+       void end_boundary(bool b) { end_boundary_ = b; }
        ///
-       bool right_boundary() const { return right_boundary_; }
+       bool end_boundary() const { return end_boundary_; }
        ///
        void flushed(bool b) { flushed_ = b; }
        ///
@@ -240,8 +243,6 @@ public:
        /// The offset of the right-most cursor position on the row
        int right_x() const;
 
-       // Return the number of separators in the row
-       int countSeparators() const;
        // Set the extra spacing for every expanding character in STRING-type
        // elements.  \param w is the total amount of extra width for the row to be
        // distributed among expanders.  \return false if the justification fails.
@@ -252,12 +253,14 @@ public:
                 Font const & f, Change const & ch);
        ///
        void add(pos_type pos, char_type const c,
-                Font const & f, Change const & ch, bool can_break);
+                Font const & f, Change const & ch);
        ///
        void addVirtual(pos_type pos, docstring const & s,
                        Font const & f, Change const & ch);
        ///
        void addSpace(pos_type pos, int width, Font const & f, Change const & ch);
+       ///
+       void addMarginSpace(pos_type pos, int width, Font const & f, Change const & ch);
 
        ///
        typedef std::vector<Element> Elements;
@@ -291,12 +294,12 @@ public:
        /**
         * if row width is too large, remove all elements after last
         * separator and update endpos if necessary. If all that
* remains is a large word, cut it to \param width.
* \param width maximum width of the row.
* \param available width on next row.
* \return true if the row has been shortened.
         */
-       bool shortenIfNeeded(int const width, int const next_width);
+       Elements shortenIfNeeded(int const max_width, int const next_width);
 
        /**
         * If last element of the row is a string, compute its width
@@ -308,10 +311,12 @@ public:
         * Find sequences of right-to-left elements and reverse them.
         * This should be called once the row is completely built.
         */
-       void reverseRTL(bool rtl_par);
+       void reverseRTL();
        ///
        bool isRTL() const { return rtl_; }
        ///
+       void setRTL(bool rtl) { rtl_ = rtl; }
+       ///
        bool needsChangeBar() const { return changebar_; }
        ///
        void needsChangeBar(bool ncb) { changebar_ = ncb; }
@@ -369,7 +374,7 @@ private:
        /// one behind last pos covered by this row
        pos_type end_ = 0;
        // Is there a boundary at the end of the row (display inset...)
-       bool right_boundary_ = false;
+       bool end_boundary_ = false;
        // Shall the row be flushed when it is supposed to be justified?
        bool flushed_ = false;
        /// Row dimension.
@@ -382,6 +387,8 @@ private:
        bool changebar_ = false;
 };
 
+std::ostream & operator<<(std::ostream & os, Row::Elements const & elts);
+
 
 /**
  * Each paragraph is broken up into a number of rows on the screen.
@@ -291,12 +294,12 @@ public:
         * if row width is too large, remove all elements after last
         * separator and update endpos if necessary. If all that
-        * remains is a large word, cut it to \param width.
-        * \param width maximum width of the row.
-        * \param available width on next row.
-        * \return true if the row has been shortened.
+        * remains is a large word, cut it to \c max_width.
+        * \param max_width maximum width of the row.
+        * \param next_width available width on next row.
+        * \return list of elements remaining after breaking.
         */